]> git.lyx.org Git - features.git/blobdiff - src/Row.h
Fix various selection-related problems
[features.git] / src / Row.h
index 03aeb746369265df318a9faff0dfc44c47d75405..7725427b9be615d286e9ba0165fea49ba1f07c52 100644 (file)
--- a/src/Row.h
+++ b/src/Row.h
@@ -30,37 +30,58 @@ class DocIterator;
 class Inset;
 
 /**
- * An on-screen row of text. A paragraph is broken into a
- * RowList for display. Each Row contains position pointers
- * into the first and last character positions of that row.
+ * An on-screen row of text. A paragraph is broken into a RowList for
+ * display. Each Row contains a tokenized description of the contents
+ * of the line.
  */
 class Row {
 public:
+       // Possible types of row elements
+       enum Type {
+               // a string of character
+               STRING,
+               /**
+                * Something (completion, end-of-par marker)
+                * that occupies space one screen but does not
+                * correspond to any paragraph contents
+                */
+               VIRTUAL,
+               // A stretchable space, basically
+               SEPARATOR,
+               // An inset
+               INSET,
+               // Some spacing described by its width, not a string
+               SPACE
+       };
+
 /**
  * One element of a Row. It has a set of attributes that can be used
  * by other methods that need to parse the Row contents.
  */
        struct Element {
-               enum Type {
-                       STRING,
-                       COMPLETION,
-                       SEPARATOR,
-                       INSET,
-                       SPACE
-               };
-
                Element(Type const t, pos_type p, Font const & f, Change const & ch)
                        : type(t), pos(p), endpos(p + 1), inset(0),
                          extra(0), font(f), change(ch), final(false) {}
 
-               //
-               bool isSeparator() const { return type == SEPARATOR; }
                // returns total width of element, including separator overhead
                double width() const { return dim.wid + extra; };
                // returns position in pixels (from the left) of position
-               // \param i in the row element
+               // \param i in the row element.
                double pos2x(pos_type const i) const;
 
+               /** Return character position that is the closest to
+                *  pixel position \param x. The value \param x is
+                *  rounded to the actual pixel position. If \param
+                *  short is true, the pixel value is rounded by
+                *  default.
+               */
+               pos_type x2pos(double &x, bool low = false) const;
+
+               // Returns the position on left side of the element.
+               pos_type left_pos() const;
+               // Returns the position on right side of the element.
+               pos_type right_pos() const;
+
                // The kind of row element
                Type type;
                // position of the element in the paragraph
@@ -112,13 +133,18 @@ public:
                DocIterator const & end) const;
 
        ///
-       void pos(pos_type p);
+       void pos(pos_type p) { pos_ = p; }
        ///
        pos_type pos() const { return pos_; }
        ///
-       void endpos(pos_type p);
+       void endpos(pos_type p) { end_ = p; }
        ///
        pos_type endpos() const { return end_; }
+       ///
+       void right_boundary(bool b) { right_boundary_ = b; }
+       ///
+       bool right_boundary() const { return right_boundary_; }
+
        ///
        Dimension const & dimension() const { return dim_; }
        ///
@@ -139,8 +165,8 @@ public:
        void add(pos_type pos, char_type const c,
                 Font const & f, Change const & ch);
        ///
-       void addCompletion(pos_type pos, docstring const & s,
-                          Font const & f, Change const & ch);
+       void addVirtual(pos_type pos, docstring const & s,
+                       Font const & f, Change const & ch);
        ///
        void addSeparator(pos_type pos, char_type const c,
                          Font const & f, Change const & ch);
@@ -165,6 +191,10 @@ public:
        ///
        bool empty() const { return elements_.empty(); }
        ///
+       Element & front() { return elements_.front(); }
+       ///
+       Element const & front() const { return elements_.front(); }
+       ///
        Element & back() { return elements_.back(); }
        ///
        Element const & back() const { return elements_.back(); }
@@ -173,11 +203,13 @@ public:
        /// remove all row elements
        void clear() { elements_.clear(); }
        /**
-        * remove all elements after last separator and update endpos
-        * if necessary.
-        * \param keep is the minimum amount of text to keep.
+        * if row width is too large, remove all elements after last
+        * separator and update endpos if necessary. If all that
+        * remains is a large word, cut it to \param width.
+        * \param body_pos minimum amount of text to keep.
+        * \param width maximum width of the row
         */
-       void separate_back(pos_type keep);
+       void shorten_if_needed(pos_type const body_pos, int const width);
 
        /**
         * If last element of the row is a string, compute its width
@@ -186,10 +218,10 @@ public:
        void finalizeLast();
 
        /**
* Find sequences of RtL elements and reverse them.
         * This should be called once the row is completely built.
         */
-       void reverseRtL();
+       void reverseRTL(bool rtl_par);
 
        friend std::ostream & operator<<(std::ostream & os, Row const & row);
 
@@ -197,8 +229,10 @@ public:
        double separator;
        /// width of hfills in the label
        double label_hfill;
/// the x position of the row
        double x;
+       /// the right margin of the row
+       int right_margin;
        ///
        mutable pos_type sel_beg;
        ///
@@ -236,6 +270,8 @@ private:
        pos_type pos_;
        /// one behind last pos covered by this row
        pos_type end_;
+       // Is there is a boundary at the end of the row (display inset...)
+       bool right_boundary_;
        /// Row dimension.
        Dimension dim_;
 };
@@ -186,10 +218,10 @@ public:
 
        /**
-        * Find sequences of RtL elements and reverse them.
+        * Find sequences of right-to-left elements and reverse them.
         * This should be called once the row is completely built.
         */
@@ -197,8 +229,10 @@ public:
        /// width of hfills in the label
        double label_hfill;
-       /// the x position of the row
+       /// the x position of the row (left margin)
        double x;